@@ -1,13 +1,12 @@
 package # hide from pause until we figure it all out
   DBIx::Class::Storage::BlockRunner;
 
-use warnings;
-use strict;
-
+use Sub::Quote 'quote_sub';
+use DBIx::Class::Exception;
 use DBIx::Class::Carp;
 use Context::Preserve 'preserve_context';
-use DBIx::Class::_Util qw( is_exception qsub dbic_internal_try dbic_internal_catch );
-use Scalar::Util qw(weaken blessed reftype);
+use Scalar::Util qw/weaken blessed/;
+use Try::Tiny;
 use Moo;
 use namespace::clean;
 
@@ -35,43 +34,60 @@ has wrap_txn => (
 has retry_handler => (
   is => 'ro',
   required => 1,
-  isa => qsub q{
-    (Scalar::Util::reftype($_[0])||'') eq 'CODE'
+  isa => quote_sub( q|
+    (ref $_[0]) eq 'CODE'
       or DBIx::Class::Exception->throw('retry_handler must be a CODE reference')
-  },
+  |),
+);
+
+has run_code => (
+  is => 'ro',
+  required => 1,
+  isa => quote_sub( q|
+    (ref $_[0]) eq 'CODE'
+      or DBIx::Class::Exception->throw('run_code must be a CODE reference')
+  |),
+);
+
+has run_args => (
+  is => 'ro',
+  isa => quote_sub( q|
+    (ref $_[0]) eq 'ARRAY'
+      or DBIx::Class::Exception->throw('run_args must be an ARRAY reference')
+  |),
+  default => quote_sub( '[]' ),
 );
 
 has retry_debug => (
   is => 'rw',
-  # use a sub - to be evaluated on the spot lazily
-  default => qsub '$ENV{DBIC_STORAGE_RETRY_DEBUG}',
-  lazy => 1,
+  default => quote_sub( '$ENV{DBIC_STORAGE_RETRY_DEBUG}' ),
 );
 
-has max_attempts => (
+has max_retried_count => (
   is => 'ro',
-  default => 20,
+  default => quote_sub( '20' ),
 );
 
-has failed_attempt_count => (
+has retried_count => (
   is => 'ro',
-  init_arg => undef,  # ensures one can't pass the value in
-  writer => '_set_failed_attempt_count',
-  default => 0,
+  init_arg => undef,
+  writer => '_set_retried_count',
+  clearer => '_reset_retried_count',
+  default => quote_sub(q{ 0 }),
   lazy => 1,
-  trigger => qsub q{
+  trigger => quote_sub(q{
     $_[0]->throw_exception( sprintf (
-      'Reached max_attempts amount of %d, latest exception: %s',
-      $_[0]->max_attempts, $_[0]->last_exception
-    )) if $_[0]->max_attempts <= ($_[1]||0);
-  },
+      'Exceeded max_retried_count amount of %d, latest exception: %s',
+      $_[0]->max_retried_count, $_[0]->last_exception
+    )) if $_[0]->max_retried_count < ($_[1]||0);
+  }),
 );
 
 has exception_stack => (
   is => 'ro',
   init_arg => undef,
   clearer => '_reset_exception_stack',
-  default => qsub q{ [] },
+  default => quote_sub(q{ [] }),
   lazy => 1,
 );
 
@@ -82,35 +98,28 @@ sub throw_exception { shift->storage->throw_exception (@_) }
 sub run {
   my $self = shift;
 
+  $self->throw_exception('run() takes no arguments') if @_;
+
   $self->_reset_exception_stack;
-  $self->_set_failed_attempt_count(0);
-
-  my $cref = shift;
-
-  $self->throw_exception('run() requires a coderef to execute as its first argument')
-    if ( reftype($cref)||'' ) ne 'CODE';
-
+  $self->_reset_retried_count;
   my $storage = $self->storage;
 
-  return $cref->( @_ ) if (
-    $storage->{_in_do_block}
-      and
-    ! $self->wrap_txn
-  );
+  return $self->run_code->( @{$self->run_args} )
+    if (! $self->wrap_txn and $storage->{_in_do_block});
 
   local $storage->{_in_do_block} = 1 unless $storage->{_in_do_block};
 
-  return $self->_run($cref, @_);
+  return $self->_run;
 }
 
 # this is the actual recursing worker
 sub _run {
-  # internal method - we know that both refs are strong-held by the
-  # calling scope of run(), hence safe to weaken everything
-  weaken( my $self = shift );
-  weaken( my $cref = shift );
+  # warnings here mean I did not anticipate some ueber-complex case
+  # fatal warnings are not warranted
+  no warnings;
+  use warnings;
 
-  my $args = @_ ? \@_ : [];
+  my $self = shift;
 
   # from this point on (defined $txn_init_depth) is an indicator for wrap_txn
   # save a bit on method calls
@@ -119,29 +128,26 @@ sub _run {
 
   my $run_err = '';
 
+  weaken (my $weakself = $self);
+
   return preserve_context {
-    dbic_internal_try {
+    try {
       if (defined $txn_init_depth) {
-        $self->storage->txn_begin;
+        $weakself->storage->txn_begin;
         $txn_begin_ok = 1;
       }
-      $cref->( @$args );
-    } dbic_internal_catch {
+      $weakself->run_code->( @{$weakself->run_args} );
+    } catch {
       $run_err = $_;
       (); # important, affects @_ below
     };
   } replace => sub {
     my @res = @_;
 
-    my $storage = $self->storage;
+    my $storage = $weakself->storage;
+    my $cur_depth = $storage->transaction_depth;
 
-    if (
-      defined $txn_init_depth
-        and
-      ! is_exception $run_err
-        and
-      defined( my $cur_depth = $storage->transaction_depth )
-    ) {
+    if (defined $txn_init_depth and $run_err eq '') {
       my $delta_txn = (1 + $txn_init_depth) - $cur_depth;
 
       if ($delta_txn) {
@@ -150,32 +156,35 @@ sub _run {
           'Unexpected reduction of transaction depth by %d after execution of '
         . '%s, skipping txn_commit()',
           $delta_txn,
-          $cref,
+          $weakself->run_code,
         ) unless $delta_txn == 1 and $cur_depth == 0;
       }
       else {
-        dbic_internal_try {
-          $storage->txn_commit;
-          1;
-        }
-        dbic_internal_catch {
-          $run_err = $_;
-        };
+        $run_err = eval { $storage->txn_commit; 1 } ? '' : $@;
       }
     }
 
     # something above threw an error (could be the begin, the code or the commit)
-    if ( is_exception $run_err ) {
+    if ($run_err ne '') {
 
-      # Attempt a rollback if we did begin in the first place
-      # Will append rollback error if possible
-      $storage->__delicate_rollback( \$run_err )
-        if $txn_begin_ok;
+      # attempt a rollback if we did begin in the first place
+      if ($txn_begin_ok) {
+        # some DBDs go crazy if there is nothing to roll back on, perform a soft-check
+        my $rollback_exception = $storage->_seems_connected
+          ? (! eval { $storage->txn_rollback; 1 }) ? $@ : ''
+          : 'lost connection to storage'
+        ;
 
-      push @{ $self->exception_stack }, $run_err;
+        if ( $rollback_exception and (
+          ! defined blessed $rollback_exception
+            or
+          ! $rollback_exception->isa('DBIx::Class::Storage::NESTED_ROLLBACK_EXCEPTION')
+        ) ) {
+          $run_err = "Transaction aborted: $run_err. Rollback failed: $rollback_exception";
+        }
+      }
 
-      # this will throw if max_attempts is reached
-      $self->_set_failed_attempt_count($self->failed_attempt_count + 1);
+      push @{ $weakself->exception_stack }, $run_err;
 
       # init depth of > 0 ( > 1 with AC) implies nesting - no retry attempt queries
       $storage->throw_exception($run_err) if (
@@ -185,47 +194,40 @@ sub _run {
           # FIXME - we assume that $storage->{_dbh_autocommit} is there if
           # txn_init_depth is there, but this is a DBI-ism
           $txn_init_depth > ( $storage->{_dbh_autocommit} ? 0 : 1 )
-        )
-          or
-        ! do {
-          local $self->storage->{_in_do_block_retry_handler} = 1
-            unless $self->storage->{_in_do_block_retry_handler};
-          $self->retry_handler->($self)
-        }
+        ) or ! $weakself->retry_handler->($weakself)
       );
 
+      $weakself->_set_retried_count($weakself->retried_count + 1);
+
       # we got that far - let's retry
-      carp( sprintf 'Retrying %s (attempt %d) after caught exception: %s',
-        $cref,
-        $self->failed_attempt_count + 1,
+      carp( sprintf 'Retrying %s (run %d) after caught exception: %s',
+        $weakself->run_code,
+        $weakself->retried_count + 1,
         $run_err,
-      ) if $self->retry_debug;
+      ) if $weakself->retry_debug;
 
       $storage->ensure_connected;
       # if txn_depth is > 1 this means something was done to the
-      # original $dbh, otherwise we would not get past the preceding if()
+      # original $dbh, otherwise we would not get past the preceeding if()
       $storage->throw_exception(sprintf
         'Unexpected transaction depth of %d on freshly connected handle',
         $storage->transaction_depth,
       ) if (defined $txn_init_depth and $storage->transaction_depth);
 
-      return $self->_run($cref, @$args);
+      return $weakself->_run;
     }
 
     return wantarray ? @res : $res[0];
   };
 }
 
-=head1 FURTHER QUESTIONS?
+=head1 AUTHOR AND CONTRIBUTORS
 
-Check the list of L<additional DBIC resources|DBIx::Class/GETTING HELP/SUPPORT>.
+See L<AUTHOR|DBIx::Class/AUTHOR> and L<CONTRIBUTORS|DBIx::Class/CONTRIBUTORS> in DBIx::Class
 
-=head1 COPYRIGHT AND LICENSE
+=head1 LICENSE
 
-This module is free software L<copyright|DBIx::Class/COPYRIGHT AND LICENSE>
-by the L<DBIx::Class (DBIC) authors|DBIx::Class/AUTHORS>. You can
-redistribute it and/or modify it under the same terms as the
-L<DBIx::Class library|DBIx::Class/COPYRIGHT AND LICENSE>.
+You may distribute this code under the same terms as Perl itself.
 
 =cut
 
